/*
 *	General functions.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <math.h>
/*#include <gif_lib.h>*/

#include "hr_defines.h"
#include "hr_colors.h"
#include "hr_bobs.h"
#include "hr_fonts.h"
#include "hr_bitmaps.h"
#include "hr_mouse.h"
#include "hr_keyboard.h"
#include "hr_zones.h"
#include "hr_buttons.h"
#include "hr_vconsole.h"
#include "hr_windows.h"
#include "hr_screens.h"
#include "hr_input_handler.h"
#include "hr_strings.h"
#include "hr_pbm.h"
#include "hr_sound.h"


/* This struct contains most HobbiRewl globals...
 * easier to hunt em down this way.*/
struct {
	unsigned int flags;
	unsigned char    *datapath;
	unsigned char    *fillpattern;
	struct hr_color  *def_colors;
	struct hr_button *firstbutton;
	struct hr_button *default_button;
	struct hr_font   *font8x8;
	struct hr_menu   *firstmenu;
	struct hr_screen *active_screen;
	GraphicsContext  *physicalscreen;
	struct hr_window *firstwindow;
	struct hr_window *active_window;
	struct hr_zone   *firstzone;
	struct hr_bob    *firstbob;
	struct hr_palette *active_palette;
   int def_hotkey;
   int (*def_func)();
	int current_screenmode;
} hr;

int pcos[360];
int psin[360];

/* Prints an error and exits. FIX! */
void hr_error_exit(unsigned char *text)
{
	if(text!=NULL)
	  printf(":\t%s\n",text);
	/* Print the error.*/
	printf(":\t%d:\t%s\n",errno,strerror(errno));
	exit(1);
}


/* Call this before exiting.*/
/* FIX! Check that everything is freed.*/
void hr_cleanup()
{
	hr_screen_close(hr.active_screen);
#if 0
	vga_setmode(TEXT);
	usleep(50000);
#endif
	hr_vc_switchback();
	hr_delete_color_struct(hr.def_colors);
#ifdef SOUND_SUPPORT
	hr_sound_close();
#endif
}



/* Always call this first.*/
void hr_init()
{
	int a;
	struct hr_bitmap *bmap;
	unsigned char tmp[HR_BUFFER_SIZE];
	/* Clear global flags.*/
	hr.flags=0;
#ifdef SOUND_SUPPORT
	/* Init sound.*/
	if(hr_sound_init()==EXIT_SUCCESS)
	  hr.flags |= HR_SOUND_INITIALIZED;
#endif
	/* Run in free vconsole.*/
	hr_get_vconsole();
	atexit(hr_cleanup);	
#if 0
	vga_safety_fork(&hr_cleanup);			/* FIX */
#endif
#if 1
	vga_disabledriverreport();				/* FIX */
#endif
	/* Give up root privileges and init svgalib stuff..*/
	vga_init();	
	if(!(hr.datapath=getenv("HOBBIREWL_DATAPATH"))) {
		printf("HOBBIREWL_DATAPATH env variable not found.\n");
		exit(0);
	}
	srandom((unsigned int)time(NULL));
	/* Precalc sin/cos tables and convert from radians to degrees.*/
	for(a=0;a<360;a++){
		psin[a]=(int)(sin((double)((a*2.0*M_PI)/360.0))*((double)(1<<SCALE)));
		pcos[a]=(int)(cos((double)((a*2.0*M_PI)/360.0))*((double)(1<<SCALE)));
	}
	/* Nullify some pointers */
	hr.fillpattern=NULL;
   hr.firstzone=NULL;
	hr.firstbutton=NULL;
	hr.firstwindow=NULL;
	hr.firstbob=NULL;
   hr.def_hotkey=0;
	hr.def_func=NULL;
	/* NULL-active window means the "root" window.*/
	hr.active_window=NULL;
	/* Allocate gfx context for physical screen and nullify
	 * active screen pointer.*/
	hr.physicalscreen=gl_allocatecontext();
	hr.active_screen=NULL;
	/* No buttons yet.*/
	hr.default_button=NULL;
	/* Default color structure. */
	hr.def_colors=NULL;   					/* This is a MUST! See func below. */
	hr.def_colors=hr_create_color_struct(7);
	hr.def_colors->index[COLOR_BACKGROUND]=0;
	hr.def_colors->index[COLOR_FOREGROUND]=1;
	hr.def_colors->index[COLOR_DARK]=2;
	hr.def_colors->index[COLOR_LIGHT]=1;
	hr.def_colors->index[COLOR_HILITE]=3;
	hr.def_colors->index[COLOR_RAISED]=4;
	hr.def_colors->index[COLOR_PRESSED]=5;
	/* Default font.*/
	sprintf(tmp,"%s%s",hr.datapath,FONT8X8);
	bmap=hr_read_pbm(tmp,GFX_MONO);
	hr.font8x8=hr_create_font(bmap,FONT_DEFAULT_INDEX,FONT_MONO);
	/* Color/palette stuff */
	hr.active_palette=hr_create_palette(NONE);
	/**/
	hr.current_screenmode=TEXT;
	/* Init mouse */
	hr_init_mouse();
	/* Clear keyboard input stream...*/
	hr_clearkey();
};




/* Ugly shit.*/
void hr_debug_log(unsigned char *text)
{
	unsigned char *cmd_line;
	cmd_line=malloc(strlen(text)+26);
	sprintf(cmd_line,"echo \"%s\" >>/tmp/debug.log\n",text);
	system(cmd_line);
	free(cmd_line);
};


	
/* 16 first colors in palette are reserved.
 * These are never changed (at least, should not be changed).
 * The color style is taken from AmigaOs 2.0 (default)
 * and modified a bit.*/
void hr_default_palette()
{
	int c;
	/* 4 AmigaOs 2.0-style colors */
	/* With ET4000 (15,15,15) was ok for color 0.
	 * CL-GD5422-80QC-D seems to output darker colors.*/
	gl_setpalettecolor(0,19,19,19);       /* Gray background. */
	gl_setpalettecolor(1,50,50,50);       /* White */
	gl_setpalettecolor(2,0,0,0);          /* Black */
	gl_setpalettecolor(3,0,10,47);       /* Blue */
	/* "pressed" and "raised" gray */
	gl_setpalettecolor(4,20,20,20);       /* Lighter Gray */
	gl_setpalettecolor(5,18,18,18);       /* Darker  Gray */
	/* 12-15 Pointer */
	hr_use_bitmap_palette(hr_mouse.bob->bitmap,12,15,NONE);
	/* Mark these colors used.*/
	for(c=0;c<16;c++)
	  hr_set_color(hr.active_palette,c,COLOR_USED);
}



/* Returns pointer to target string containing percentage info */
/* FIX! Often does unnecessary strcpy'ing... */
unsigned char *hr_percentage_string(unsigned char *target,
												int min,int max,int def,int val,
												unsigned char *minstring,
												unsigned char *maxstring,
												unsigned char *defstring)
{
	sprintf(target,"%d%%",(val*100)/max);
	if(val==min){
		if(minstring!=NULL)
		  strcpy(target,minstring);
	}
	if(val==max){
		if(maxstring!=NULL)
		  strcpy(target,maxstring);
	}
	if(val==def){
		if(defstring!=NULL)
		  strcpy(target,defstring);
	}
	return target;
}


/* Replaces "what"-chars in "where"-string with "with"-chars.*/
void hr_replace_chars(unsigned char *where,
							 unsigned char *what,unsigned char *with)
{
	int c,l;
	if((where!=NULL)&&(what!=NULL)&&(with!=NULL)){
		/* "what" and "with"-strings must be of equal size.*/
		if(strlen(what)==strlen(with)){
			/* Scan whole "where"-string.*/
			for(c=0;c<strlen(where);c++){
				/* Compare with all "what"-chars.*/
				for(l=0;l<strlen(what);l++){
					/* Replace chars */
					if(*(where+c)==*(what+l)){
						*(where+c)=*(with+l);
						break;
					}
				}
			}
		}
	}
}


/* Returns direction angle of vector (dx,dy) in radians.
 * dy grows upwards on screen. */
double hr_return_angle(double dx,double dy)
{
	double angle;
	angle=atan2(dy,dx);
	if(angle<0)
	  angle+=(2*M_PI);
	return angle;
}


/* A Simple Yes/No-requester */
int hr_ask_confirmation(unsigned char *question)
{
	int rval=0;
	int w;
	struct hr_window *win;
	struct hr_button *b;
	w=(strlen(question)*8+20);
	win=hr_window_open((WIDTH/2)-(w/2),(HEIGHT/2)-20,w,40,
							 hr.def_colors,WINDOW_CLIPPING);
	/**/
	win->gadgets=hr_create_window_gadgets(2);
	(win->gadgets->button)[0]=
	  hr_create_button(0,0,0,0,
							 hr.font8x8,hr.def_colors,
							 "~uY~ues",NULL,'y',
							 BUTTON_ENABLED|BUTTON_INVISIBLE);
	hr.default_button=(win->gadgets->button)[1]=
	  hr_create_button(0,0,0,0,
							 hr.font8x8,hr.def_colors,
							 "~uN~uo",NULL,'n',
							 BUTTON_ENABLED|BUTTON_INVISIBLE);
	hr_window_refresh(win);
	HR_CENTER(WIDTH/2,(win->zone->y1)+9,1,0,question,hr.font8x8,NONE);
	/**/
	hr_clearkey();
	b=NULL;
	while(b==NULL)
	  b=hr_input_handler(hr_input_handler(NULL,NULL,LMB_PRESSED|KBD_PRESSED),
								NULL,AMB_RELEASED);
	if(b==(win->gadgets->button)[0])
	  rval=1;
	hr_window_close(win);
	return rval;
}
